package cn.wzpmc.hny.commands.lobby;

import cn.wzpmc.hny.commands.lobby.arguments.AspectArgumentType;
import cn.wzpmc.hny.commands.lobby.sub.CancelVoteCommand;
import cn.wzpmc.hny.commands.lobby.provider.GameSuggestionProvider;
import cn.wzpmc.hny.commands.lobby.sub.debug.DoneCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.SaveCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.get.GetCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.ResetCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.ShowCommand;
import cn.wzpmc.hny.commands.lobby.sub.VoteCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.get.GetToInventoryCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.wind.CreateWindCommand;
import cn.wzpmc.hny.commands.lobby.sub.debug.wind.RemoveWindCommand;
import cn.wzpmc.hny.commands.lobby.sub.gamble.GambleStartCommand;
import cn.wzpmc.hny.commands.lobby.sub.gamble.GambleVoteCommand;
import cn.wzpmc.hny.enums.Aspects;
import cn.wzpmc.hny.pojo.CustomItem;
import cn.wzpmc.hny.utils.Wind;
import com.mojang.brigadier.arguments.DoubleArgumentType;
import com.mojang.brigadier.arguments.IntegerArgumentType;
import com.mojang.brigadier.arguments.StringArgumentType;
import com.mojang.brigadier.builder.LiteralArgumentBuilder;
import com.mojang.brigadier.builder.RequiredArgumentBuilder;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.bukkit.permissions.ServerOperator;
import org.mmga.utils.interfaces.MmgaCommand;

public class LobbyTestCommand implements MmgaCommand {
    public LiteralArgumentBuilder<CommandSender> getVoteCommands() {
        return LiteralArgumentBuilder.<CommandSender>literal("vote")
                .then(getVoteArgumentsCommands())
                .then(getVoteCancelCommands());
    }
    public RequiredArgumentBuilder<CommandSender, String> getVoteArgumentsCommands(){
        return RequiredArgumentBuilder.<CommandSender, String>argument("stageName", StringArgumentType.word())
                .suggests(new GameSuggestionProvider())
                .executes(new VoteCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getVoteCancelCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("cancel")
                .requires(ServerOperator::isOp)
                .executes(new CancelVoteCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getGambleCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("gamble")
                .then(getGambleVoteCommands())
                .then(getGambleStartCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getGambleVoteCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("vote").then(getGambleAspectCommands());
    }
    public RequiredArgumentBuilder<CommandSender, Aspects> getGambleAspectCommands(){
        return RequiredArgumentBuilder.<CommandSender, Aspects>argument("aspect", new AspectArgumentType())
                .executes(new GambleVoteCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getGambleStartCommand(){
        return LiteralArgumentBuilder.<CommandSender>literal("start")
                .executes(new GambleStartCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getDebugCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("debug")
                .requires(ServerOperator::isOp)
                .then(this.getDebugShowCommands())
                .then(this.getDebugResetCommands())
                .then(this.getDebugDoneCommands())
                .then(this.getDebugWindCommands())
                .then(this.getDebugGetCommands())
                .then(this.getDebugSaveCommands());
    }
    public LiteralArgumentBuilder<CommandSender> getDebugSaveCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("save").executes(new SaveCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getDebugGetCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("get").then(
                RequiredArgumentBuilder.<CommandSender, Integer>argument("itemId", IntegerArgumentType.integer(-1, CustomItem.ItemStatic.items.size() - 1))
                        .executes(new GetCommand())
                        .then(LiteralArgumentBuilder.<CommandSender>literal("inventory")
                                .executes(new GetToInventoryCommand())));
    }
    public LiteralArgumentBuilder<CommandSender> getDebugShowCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("show").then(
                RequiredArgumentBuilder.<CommandSender, String>argument("stageName", StringArgumentType.word())
                        .suggests(new GameSuggestionProvider())
                        .executes(new ShowCommand()));
    }
    public LiteralArgumentBuilder<CommandSender> getDebugResetCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("reset").executes(new ResetCommand());
    }
    public LiteralArgumentBuilder<CommandSender> getDebugDoneCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("done").then(
                RequiredArgumentBuilder.<CommandSender, String>argument("stageName", StringArgumentType.word())
                        .suggests(new GameSuggestionProvider())
                        .executes(new DoneCommand()));
    }
    public LiteralArgumentBuilder<CommandSender> getDebugWindCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("wind")
                .then(this.getDebugWindCreateCommands())
                .then(this.getDebugWindRemoveCommands());
    }
    public LiteralArgumentBuilder<CommandSender> getDebugWindCreateCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("create").then(
                RequiredArgumentBuilder.<CommandSender, Double>argument("radius", DoubleArgumentType.doubleArg(0,10)).then(
                        RequiredArgumentBuilder.<CommandSender, String>argument("name", StringArgumentType.word())
                                .executes(new CreateWindCommand())));
    }
    public LiteralArgumentBuilder<CommandSender> getDebugWindRemoveCommands(){
        return LiteralArgumentBuilder.<CommandSender>literal("remove").then(
                RequiredArgumentBuilder.<CommandSender, String>argument("name", StringArgumentType.word())
                        .suggests(new Wind.WindSuggestion())
                        .executes(new RemoveWindCommand()));
    }
    @Override
    public LiteralArgumentBuilder<CommandSender> getCommandBuilder() {
        return LiteralArgumentBuilder.<CommandSender>literal("lobby-test")
                .requires(e -> e instanceof Player)
                .then(getVoteCommands())
                .then(getGambleCommands())
                .then(getDebugCommands());
    }
}
