package cn.wzpmc.hny.games;

import cn.wzpmc.hny.HappyNewLobby;
import cn.wzpmc.hny.errors.WorldCreateException;
import cn.wzpmc.hny.games.enums.Games;
import cn.wzpmc.hny.pojo.RankablePlayerData;
import cn.wzpmc.hny.utils.BetterBukkitRunnable;
import cn.wzpmc.hny.utils.MessageBroadcast;
import com.onarandombox.MultiverseCore.MultiverseCore;
import com.onarandombox.MultiverseCore.api.MVWorldManager;
import com.onarandombox.MultiverseCore.api.MultiverseWorld;
import lombok.Getter;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextDecoration;
import net.kyori.adventure.title.Title;
import org.bukkit.*;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.plugin.PluginManager;

import java.util.*;

public class DigGame implements AutoCloseable {
    public static final List<DigGame> games = new ArrayList<>();
    private final Server server;
    private final MultiverseCore multiverseCore;
    @Getter
    private final MultiverseWorld playWorld;
    private final String playWorldName;
    private final Location playerJoinLocation;
    private final HappyNewLobby plugin;
    @Getter
    private final Map<Player, RankablePlayerData> players = new HashMap<>();
    @Getter
    private boolean closed = false;
    public DigGame() throws WorldCreateException {
        this.plugin = HappyNewLobby.getPlugin(HappyNewLobby.class);
        this.server = this.plugin.getServer();
        PluginManager pluginManager = this.server.getPluginManager();
        this.multiverseCore = (MultiverseCore) pluginManager.getPlugin("Multiverse-Core");
        assert multiverseCore != null;
        MVWorldManager mvWorldManager = this.multiverseCore.getMVWorldManager();
        this.playWorldName = "dig-game-" + UUID.randomUUID();
        if (!mvWorldManager.cloneWorld("digGame", this.playWorldName)) {
            throw new WorldCreateException(this.playWorldName);
        }
        this.playWorld = mvWorldManager.getMVWorld(this.playWorldName);
        this.playWorld.setAllowAnimalSpawn(false);
        this.playWorld.setAllowMonsterSpawn(false);
        this.playWorld.setHunger(false);
        this.playWorld.setTime("day");
        this.playWorld.setDifficulty(Difficulty.PEACEFUL);
        this.playWorld.setPVPMode(true);
        this.playWorld.setEnableWeather(false);
        World cbWorld = this.playWorld.getCBWorld();
        cbWorld.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        cbWorld.setGameRule(GameRule.DO_MOB_SPAWNING, false);
        cbWorld.setGameRule(GameRule.DO_WEATHER_CYCLE, false);
        this.playerJoinLocation = new Location(cbWorld, 15, 76, 15);
        this.playWorld.setSpawnLocation(this.playerJoinLocation);
        DigGame.games.add(this);
    }
    private static final List<Component> LORE = new ArrayList<>();
    static {
        LORE.add(Component.text("特质的小金铲").color(NamedTextColor.GREEN).decoration(TextDecoration.ITALIC, false));
        LORE.add(Component.text("你可以用它打人、往外撇，甚至可以铲雪！").color(NamedTextColor.WHITE).decoration(TextDecoration.ITALIC, false));
        LORE.add(Component.text("撇出去不见了我们不负责哦~").color(NamedTextColor.DARK_RED).decorate(TextDecoration.UNDERLINED).decorate(TextDecoration.BOLD));
    }
    private static final Component SHOVEL_DISPLAY_NAME = Component.text("特质的小金铲~").color(NamedTextColor.RED).decoration(TextDecoration.ITALIC, false);
    public void prepare(){
        Collection<? extends Player> onlinePlayers = this.server.getOnlinePlayers();
        for (Player onlinePlayer : onlinePlayers){
            onlinePlayer.teleport(this.playerJoinLocation);
            PlayerInventory inventory = onlinePlayer.getInventory();
            inventory.clear();
            ItemStack itemStack = new ItemStack(Material.GOLDEN_SHOVEL, 1);
            ItemMeta itemMeta = itemStack.getItemMeta();
            itemMeta.addItemFlags(ItemFlag.HIDE_UNBREAKABLE);
            itemMeta.setUnbreakable(true);
            itemMeta.displayName(SHOVEL_DISPLAY_NAME);
            itemMeta.lore(LORE);
            itemMeta.addEnchant(Enchantment.DIG_SPEED, 5, false);
            itemStack.setItemMeta(itemMeta);
            inventory.addItem(itemStack);
            this.players.put(onlinePlayer, new RankablePlayerData());
            onlinePlayer.setGameMode(GameMode.ADVENTURE);
        }
        new MessageBroadcast(Title.title(Component.text("相信你已经准备好了吧？"), Component.empty()), onlinePlayers).runTaskLater(this.plugin, 80);
        new MessageBroadcast(Title.title(Component.text("找一个点位站好哦"), Component.empty()), onlinePlayers).runTaskLater(this.plugin, 120);
        new BetterBukkitRunnable(this::start).runTaskLater(this.plugin, 160);
    }

    public void start(){
        for (Player player : this.players.keySet()){
            player.setGameMode(GameMode.SURVIVAL);
            player.showTitle(Title.title(Component.text("游戏开始！").color(NamedTextColor.GREEN), Component.empty()));
        }
    }
    @Override
    public void close(){
        if (this.closed){
            return;
        }
        this.closed = true;
        multiverseCore.deleteWorld(this.playWorldName);
    }
    public static boolean isSpecialShovel(ItemStack stack){
        ItemMeta itemMeta = stack.getItemMeta();
        return Objects.equals(itemMeta.displayName(), DigGame.SHOVEL_DISPLAY_NAME);
    }
    public static DigGame getPlayerGame(Player player){
        for (DigGame game : DigGame.games) {
            if (game.players.containsKey(player)){
                return game;
            }
        }
        return null;
    }
    public static boolean isPlayerInGame(Player player){
        return getPlayerGame(player) != null;
    }
    public void losePlayer(Player player){
        if (this.players.get(player).getRank() != -1) {
            return;
        }
        int minRank = -1;
        Player noRankPlayer = null;
        for (Map.Entry<Player, RankablePlayerData> playerPlayerDataEntry : this.players.entrySet()) {
            Player key = playerPlayerDataEntry.getKey();
            RankablePlayerData value = playerPlayerDataEntry.getValue();
            int rank = value.getRank();
            if (rank == -1){
                if (player.equals(key)){
                    continue;
                }
                noRankPlayer = key;
                continue;
            }
            if (rank < minRank){
                minRank = rank;
            }
        }
        if (minRank == -1){
            minRank = this.players.size();
        }else {
            minRank -= 1;
        }
        RankablePlayerData digGamePlayerData = this.players.get(player);
        digGamePlayerData.setRank(minRank);
        this.players.put(player, digGamePlayerData);
        PlayerInventory inventory = player.getInventory();
        inventory.clear();
        player.showTitle(Title.title(Component.text("你死了").color(NamedTextColor.RED), Component.text("等着游戏结束吧~")));
        player.setGameMode(GameMode.SPECTATOR);
        player.teleport(this.playerJoinLocation);
        if (minRank == 2){
            if (noRankPlayer == null){
                System.out.println("ERROR NO RANK PLAYER NULL");
                return;
            }
            RankablePlayerData gamePlayerData = this.players.get(noRankPlayer);
            gamePlayerData.setRank(1);
            this.players.put(noRankPlayer, gamePlayerData);
            noRankPlayer.showTitle(Title.title(Component.text("你赢了！").color(NamedTextColor.RED), Component.empty()));
            GameStageManager.getInstance().triggerGameEnd(Games.GAME3);
        }
    }
}
