package cn.wzpmc.hny.games;

import cn.wzpmc.hny.games.enums.GameStages;
import cn.wzpmc.hny.games.enums.Games;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Server;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.mmga.utils.utils.map.IncreaseAbleHashMap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class GameStageManager {
    private static GameStageManager instance;
    private final Map<Games, Boolean> isUnlock;
    private final Map<Games, GameStages> stages;
    private final IncreaseAbleHashMap<Games, Player> votes;
    private GameStageManager(){
        this.isUnlock = new HashMap<>();
        this.stages = new HashMap<>();
        this.votes = new IncreaseAbleHashMap<>();
        for (Games value : Games.values()) {
            boolean unlock = false;
            if (value.name().equalsIgnoreCase("game1")) {
                unlock = true;
            }
            this.isUnlock.put(value, unlock);
        }
    }
    public static GameStageManager getInstance() {
        if (instance == null){
            instance = new GameStageManager();
        }
        return instance;
    }
    public static void reset(){
        instance = null;
    }
    public boolean isUnlock(Games game){
        return isUnlock.get(game);
    }
    public void setUnlock(Games game, boolean unlock){
        isUnlock.put(game, unlock);
    }
    private boolean isStage(Games game, GameStages stage){
        return stages.getOrDefault(game, GameStages.NOT_START).equals(stage);
    }
    public boolean isStart(Games game){
        return isStage(game, GameStages.STARTED);
    }
    public boolean isEnd(Games game){
        return isStage(game, GameStages.END);
    }
    public boolean isNotStart(Games game){
        return isStage(game, GameStages.NOT_START);
    }
    public void setStage(Games game, GameStages stage){
        stages.put(game, stage);
    }
    public void vote(Games game, Player player){
        if (!this.isUnlock(game)){
            player.sendMessage(Component.text("游戏未解锁，无法投票开始！").color(NamedTextColor.RED));
            return;
        }
        if (this.isNotStart(game)){
            if (votes.getOrDefault(game, new ArrayList<>()).contains(player)){
                player.sendMessage(Component.text("你已经投过票了！").color(NamedTextColor.RED));
                return;
            }
            votes.add(game, player);
            return;
        }
        player.sendMessage(Component.text("游戏已经开始，无法投票开始！").color(NamedTextColor.RED));
    }
    public List<Player> getVotePlayers(Games game){
        return votes.getOrDefault(game, new ArrayList<>());
    }

    public void cancelVote(CommandSender source) {
        Server server = source.getServer();
        for (Player onlinePlayer : server.getOnlinePlayers()) {
            onlinePlayer.sendMessage((Component.text("玩家")
                    .append(Component.text(source.getName()))
                    .append(Component.text("取消了投票！"))).color(NamedTextColor.RED));
        }
        for (Map.Entry<Games, List<Player>> gamesListEntry : votes.entrySet()) {
            gamesListEntry.setValue(new ArrayList<>());
        }
    }
    public void triggerGameStart(Games game){
        this.setStage(game, GameStages.STARTED);
        game.triggerStartEvent();
    }
    public void triggerGameEnd(Games game){
        this.setStage(game, GameStages.END);
        game.triggerDoneEvent();
    }
}
