package cn.wzpmc.hny.games;

import cn.wzpmc.hny.HappyNewLobby;
import cn.wzpmc.hny.games.enums.Games;
import cn.wzpmc.hny.games.enums.Teams;
import cn.wzpmc.hny.pojo.CustomItem;
import cn.wzpmc.hny.pojo.CustomItemInfo;
import cn.wzpmc.hny.pojo.GlobalPlayerData;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.Server;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryType;
import org.bukkit.inventory.Inventory;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class PlayerDataManager {
    private final static NamespacedKey moneyKey = new NamespacedKey("hny", "money");
    private final static NamespacedKey itemKey = new NamespacedKey("hny", "items");
    private final static NamespacedKey itemCountKey = new NamespacedKey("hny", "item_count");
    private final static NamespacedKey armorHeadKey = new NamespacedKey("hny", "armor_head");
    private final static NamespacedKey armorLegKey = new NamespacedKey("hny", "armor_leg");
    private final static NamespacedKey erosionKey = new NamespacedKey("hny", "erosion");
    private final static NamespacedKey teamKey = new NamespacedKey("hny", "team");
    private final static NamespacedKey shieldKey = new NamespacedKey("hny", "team");
    private static PlayerDataManager instance;
    private final Map<Player, GlobalPlayerData> players;
    private final Server server;
    private PlayerDataManager(){
        this.players = new HashMap<>();
        this.server = HappyNewLobby.getPlugin(HappyNewLobby.class).getServer();
    }
    public void initPlayerData(Player player){
        PersistentDataContainer persistentDataContainer = player.getPersistentDataContainer();
        Double money = persistentDataContainer.get(moneyKey, PersistentDataType.DOUBLE);
        if (money == null){
            money = 0d;
        }
        int[] itemIds = persistentDataContainer.get(itemKey, PersistentDataType.INTEGER_ARRAY);
        if (itemIds == null){
            itemIds = new int[]{};
        }
        int[] itemCounts = persistentDataContainer.get(itemCountKey, PersistentDataType.INTEGER_ARRAY);
        if (itemCounts == null){
            itemCounts = new int[]{};
        }
        List<CustomItemInfo> items = new ArrayList<>();
        for (int i = 0; i < itemIds.length; i++) {
            int itemId = itemIds[i];
            int itemCount = itemCounts[i];
            items.add(new CustomItemInfo(CustomItem.ItemStatic.getItem(itemId), itemCount));
        }
        Integer headLevel = persistentDataContainer.get(armorHeadKey, PersistentDataType.INTEGER);
        if (headLevel == null){
            headLevel = 0;
        }
        Integer legLevel = persistentDataContainer.get(armorLegKey, PersistentDataType.INTEGER);
        if (legLevel == null){
            legLevel = 0;
        }
        Double erosionLevel = persistentDataContainer.get(erosionKey, PersistentDataType.DOUBLE);
        if (erosionLevel == null){
            erosionLevel = 0d;
        }
        Teams team = Teams.valueOf(persistentDataContainer.get(teamKey, PersistentDataType.INTEGER));
        Integer rawShieldData = persistentDataContainer.get(shieldKey, PersistentDataType.INTEGER);
        if (rawShieldData == null){
            rawShieldData = 0;
        }
        boolean shield = rawShieldData == 1;
        GlobalPlayerData globalPlayerData = new GlobalPlayerData(money, items, headLevel, legLevel, erosionLevel, team, shield);
        this.players.put(player, globalPlayerData);
    }
    public GlobalPlayerData getPlayerData(Player player) {
        return this.players.get(player);
    }
    public static final Component PLAYER_STATISTICAL_INVENTORY_TITLE = Component.text("统计数据").color(NamedTextColor.RED);
    public static final Component PLAYER_INVENTORY_TITLE = Component.text("你的道具仓库").color(NamedTextColor.DARK_RED);
    public static final Component ACCOUNT_BUTTON_DISPLAY_NAME = Component.text("你的账户").color(NamedTextColor.GREEN);
    public void showPlayerData(Player player){
        Inventory inventory = this.server.createInventory(player, InventoryType.CHEST, PLAYER_STATISTICAL_INVENTORY_TITLE);
        GlobalPlayerData globalPlayerData = this.players.get(player);
        double money = globalPlayerData.getMoney();
        CustomItem moneyItem = new CustomItem(Material.EMERALD, ACCOUNT_BUTTON_DISPLAY_NAME).lore(Component.text("当前金额：").color(NamedTextColor.GOLD).append(Component.text(money)));
        GameStageManager gameStageManager = GameStageManager.getInstance();
        if (gameStageManager.isStart(Games.GAME5)) {
            moneyItem.lore(Component.text("当前队伍：").color(NamedTextColor.LIGHT_PURPLE).append(Component.text(globalPlayerData.getTeam().getName())));
            moneyItem.lore(Component.text("当前侵蚀度：").color(NamedTextColor.DARK_GREEN).append(Component.text(globalPlayerData.getErosionLevel())));
            moneyItem.lore(Component.text("当前头盔、靴子等级：").color(NamedTextColor.AQUA).append(Component.text(globalPlayerData.getArmorHeadLevel())));
            moneyItem.lore(Component.text("当前胸甲、护腿等级：").color(NamedTextColor.AQUA).append(Component.text(globalPlayerData.getArmorLegLevel())));
            moneyItem.lore(Component.text("当前是否拥有盾牌：").color(NamedTextColor.AQUA).append(Component.text(globalPlayerData.isShield() ? "有" : "没有")));
        }
        inventory.setItem(11, moneyItem.getStack(1));
        inventory.setItem(15, CustomItem.ProjectItemStatic.INTO_INVENTORY_BUTTON.getStack(1));
        player.openInventory(inventory);
    }
    public void showPlayerInventory(Player player){
        Inventory inventory = this.server.createInventory(player, InventoryType.CHEST, PLAYER_INVENTORY_TITLE);
        GlobalPlayerData globalPlayerData = this.players.get(player);
        for (CustomItemInfo customItem : globalPlayerData.getInventory()) {
            inventory.addItem(customItem.getType().getStack(customItem.getAmount()));
        }
        inventory.setItem(26, CustomItem.ProjectItemStatic.INVENTORY_BACK_BUTTON.getStack(1));
        player.openInventory(inventory);
    }
    public static PlayerDataManager getInstance(){
        if (instance == null){
            instance = new PlayerDataManager();
        }
        return instance;
    }

    public void resetAllData() {
        players.clear();
        for (Player onlinePlayer : server.getOnlinePlayers()) {
            PersistentDataContainer persistentDataContainer = onlinePlayer.getPersistentDataContainer();
            persistentDataContainer.remove(moneyKey);
            persistentDataContainer.remove(itemKey);
            persistentDataContainer.remove(itemCountKey);
            persistentDataContainer.remove(armorLegKey);
            persistentDataContainer.remove(armorHeadKey);
            persistentDataContainer.remove(erosionKey);
            persistentDataContainer.remove(teamKey);
            persistentDataContainer.remove(shieldKey);
            persistentDataContainer.set(moneyKey, PersistentDataType.DOUBLE, 0d);
            persistentDataContainer.set(itemKey, PersistentDataType.INTEGER_ARRAY, new int[0]);
            persistentDataContainer.set(itemCountKey, PersistentDataType.INTEGER_ARRAY, new int[0]);
            persistentDataContainer.set(armorLegKey, PersistentDataType.INTEGER, 0);
            persistentDataContainer.set(armorHeadKey, PersistentDataType.INTEGER, 0);
            persistentDataContainer.set(erosionKey, PersistentDataType.DOUBLE, 0d);
            persistentDataContainer.set(teamKey, PersistentDataType.INTEGER, Teams.ATTACK.getId());
            persistentDataContainer.set(shieldKey, PersistentDataType.INTEGER, 0);
            this.initPlayerData(onlinePlayer);
        }
    }
    public void savePlayerData(Player player) {
        GlobalPlayerData globalPlayerData = this.players.get(player);
        List<CustomItemInfo> inventory = globalPlayerData.getInventory();
        double money = globalPlayerData.getMoney();
        int[] ids = new int[inventory.size()];
        int[] counts = new int[inventory.size()];
        int i = 0;
        for (CustomItemInfo customItem : inventory) {
            Integer id = CustomItem.ItemStatic.getId(customItem.getType());
            counts[i] = customItem.getAmount();
            ids[i++] = id;
        }
        PersistentDataContainer persistentDataContainer = player.getPersistentDataContainer();
        persistentDataContainer.set(moneyKey, PersistentDataType.DOUBLE, money);
        persistentDataContainer.set(itemKey, PersistentDataType.INTEGER_ARRAY, ids);
        persistentDataContainer.set(itemCountKey, PersistentDataType.INTEGER_ARRAY, counts);
        persistentDataContainer.set(armorHeadKey, PersistentDataType.INTEGER, globalPlayerData.getArmorHeadLevel());
        persistentDataContainer.set(armorLegKey, PersistentDataType.INTEGER, globalPlayerData.getArmorLegLevel());
        persistentDataContainer.set(erosionKey, PersistentDataType.DOUBLE, globalPlayerData.getErosionLevel());
        persistentDataContainer.set(teamKey,  PersistentDataType.INTEGER, globalPlayerData.getTeam().getId());
        persistentDataContainer.set(shieldKey,  PersistentDataType.INTEGER, globalPlayerData.isShield() ? 1 : 0);
    }
}
