package cn.wzpmc.hny.games;

import cn.wzpmc.hny.utils.IncreaseAbleConcurrentHashMap;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.Getter;
import org.bukkit.entity.Player;
import org.mmga.utils.utils.map.IncreaseAbleHashMap;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.TimeUnit;
import java.util.function.Predicate;

public class PlayerEffectManager {
    public enum Effects {
        RAINBOW_HEALTH,
        POWER_WELDING,
        BADLY_HURT,
        BE_BADLY_HURT,
        TOTEM_OF_UNDYING,
        DISH,
        TREAT,
        PUT_OFF,
        ERROR,
        CODE,
        TURBINE,
        MARK
    }
    @EqualsAndHashCode
    private static class EffectInfo {
        @Getter
        private final Effects type;
        @Getter
        private long tickTimes;
        private boolean hidden;
        private final Map<String, String> attr;
        public EffectInfo(Effects type, long tickTimes){
            this.type =  type;
            this.tickTimes = tickTimes;
            this.hidden = false;
            this.attr = new HashMap<>();
        }
        public long onTick(){
            return this.tickTimes--;
        }
        public boolean isHidden(){
            if (hidden){
                hidden = false;
                return true;
            }
            return false;
        }
        public void hidden(){
            this.hidden = true;
        }
        public void setAttr(String k, Object v){
            this.attr.put(k, v.toString());
        }
        public String getAttr(String k){
            return this.attr.get(k);
        }
    }
    private static final IncreaseAbleConcurrentHashMap<Player, EffectInfo> effectInfoMap = new IncreaseAbleConcurrentHashMap<>();
    private static PlayerEffectManager instance;
    private PlayerEffectManager(){}
    public static PlayerEffectManager getInstance(){
        if (instance == null){
            instance = new PlayerEffectManager();
        }
        return instance;
    }
    public boolean hasEffect(Player player, Effects effect){
        return effectInfoMap.getOrDefault(player, new CopyOnWriteArrayList<>()).stream().anyMatch(e -> e.getType().equals(effect) && !e.isHidden());
    }
    public String getEffectAttr(Player player, Effects effect, String k){
        for (EffectInfo effectInfo : effectInfoMap.get(player)) {
            if (effectInfo.getType().equals(effect)){
                return effectInfo.getAttr(k);
            }
        }
        return null;
    }
    public void setEffectAttr(Player player, Effects effect, String k, Object v){
        for (EffectInfo effectInfo : effectInfoMap.get(player)) {
            if (effectInfo.getType().equals(effect)){
                effectInfo.setAttr(k, v);
                break;
            }
        }
    }
    public void addEffect(Player player, Effects effect, long time, TimeUnit unit){
        this.addEffect(player, effect, unit.toSeconds(time) * 20);
    }
    public void addEffect(Player player, Effects effect, long ticks){
        effectInfoMap.add(player, new EffectInfo(effect, ticks));
    }
    public void removeEffect(Player player, Effects effects){
        List<EffectInfo> effectInfos = effectInfoMap.get(player);
        effectInfos.removeIf(e -> e.getType().equals(effects));
    }
    public void removeEffect(Player player){
        effectInfoMap.put(player, new CopyOnWriteArrayList<>());
    }
    public void hideEffect(Player player, Effects effects){
        List<EffectInfo> effectInfos = effectInfoMap.get(player);
        for (EffectInfo effectInfo : effectInfos) {
            if (effectInfo.getType().equals(effects)){
                effectInfo.hidden();
            }
        }
    }

    public void onTick(){
        effectInfoMap.forEach((key, value) -> value.forEach(eff -> {
            long l = eff.onTick();
            if (l == 0) {
                effectInfoMap.delete(key, eff);
            }
        }));
    }
}
