package cn.wzpmc.hny.games;

import cn.wzpmc.hny.HappyNewLobby;
import cn.wzpmc.hny.errors.WorldCreateException;
import cn.wzpmc.hny.games.enums.Games;
import cn.wzpmc.hny.pojo.GlobalPlayerData;
import cn.wzpmc.hny.pojo.RankablePlayerData;
import cn.wzpmc.hny.utils.BetterBukkitRunnable;
import cn.wzpmc.hny.utils.MessageBroadcast;
import com.onarandombox.MultiverseCore.MultiverseCore;
import com.onarandombox.MultiverseCore.api.MVWorldManager;
import com.onarandombox.MultiverseCore.api.MultiverseWorld;
import lombok.Getter;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.title.Title;
import org.bukkit.*;
import org.bukkit.enchantments.Enchantment;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemFlag;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.plugin.PluginManager;

import java.util.*;

public class SumoGame implements AutoCloseable{
    public static final List<SumoGame> games = new ArrayList<>();
    private final Server server;
    private final MultiverseCore multiverseCore;
    @Getter
    private final MultiverseWorld playWorld;
    private final String playWorldName;
    private final Location playerJoinLocation;
    private final Location aSpawn;
    private final Location bSpawn;
    private final HappyNewLobby plugin;
    @Getter
    private final Map<Player, RankablePlayerData> players = new HashMap<>();
    @Getter
    private boolean closed = false;
    @Getter
    private Player playerA;
    @Getter
    private Player playerB;
    private int nowPrice = 0;
    private int nowRank = 1;
    private boolean isWaiting = false;
    public SumoGame() throws WorldCreateException {
        this.plugin = HappyNewLobby.getPlugin(HappyNewLobby.class);
        this.server = this.plugin.getServer();
        PluginManager pluginManager = this.server.getPluginManager();
        this.multiverseCore = (MultiverseCore) pluginManager.getPlugin("Multiverse-Core");
        assert multiverseCore != null;
        MVWorldManager mvWorldManager = this.multiverseCore.getMVWorldManager();
        this.playWorldName = "sumo-game-" + UUID.randomUUID();
        if (!mvWorldManager.cloneWorld("sumoGame", this.playWorldName)) {
            throw new WorldCreateException(this.playWorldName);
        }
        this.playWorld = mvWorldManager.getMVWorld(this.playWorldName);
        this.playWorld.setAllowAnimalSpawn(false);
        this.playWorld.setAllowMonsterSpawn(false);
        this.playWorld.setHunger(false);
        this.playWorld.setTime("day");
        this.playWorld.setDifficulty(Difficulty.PEACEFUL);
        this.playWorld.setEnableWeather(false);
        World cbWorld = this.playWorld.getCBWorld();
        cbWorld.setGameRule(GameRule.DO_DAYLIGHT_CYCLE, false);
        cbWorld.setGameRule(GameRule.DO_MOB_SPAWNING, false);
        this.playerJoinLocation = new Location(cbWorld, 6, 71, 6);
        this.aSpawn = new Location(cbWorld, 6, 65, 11, 180, 0);
        this.bSpawn = new Location(cbWorld, 6, 65, 2, 0, 0);
        this.playWorld.setSpawnLocation(this.playerJoinLocation);
        SumoGame.games.add(this);
    }
    public void prepare(){
        for (Player onlinePlayer : this.server.getOnlinePlayers()) {
            onlinePlayer.setGameMode(GameMode.SPECTATOR);
            onlinePlayer.teleport(this.playerJoinLocation);
            PlayerInventory inventory = onlinePlayer.getInventory();
            inventory.clear();
            this.players.put(onlinePlayer, new RankablePlayerData());
            nowRank += 1;
        }
        round();
    }
    public void round(){
        Collection<? extends Player> onlinePlayers = this.server.getOnlinePlayers();
        List<Player> allIn = new ArrayList<>();
        for (Map.Entry<Player, RankablePlayerData> playerRankablePlayerDataEntry : this.players.entrySet()) {
            Player key = playerRankablePlayerDataEntry.getKey();
            if (playerRankablePlayerDataEntry.getValue().getRank() == -1) {
                allIn.add(key);
            }
        }
        if (allIn.size() == 1){
            Player winPlayer = allIn.get(0);
            PlayerDataManager instance = PlayerDataManager.getInstance();
            instance.getPlayerData(winPlayer).addMoney(nowPrice);
            instance.savePlayerData(winPlayer);
            GameStageManager.getInstance().triggerGameEnd(Games.GAME1);
            return;
        }
        Collections.shuffle(allIn);
        new MessageBroadcast(Title.title(Component.text("正在抽取幸运儿...").color(NamedTextColor.GOLD), Component.empty()), onlinePlayers).runTaskLater(this.plugin, 40);
        this.playerA = allIn.get(0);
        this.playerB = allIn.get(1);
        new MessageBroadcast(Title.title(this.playerA.displayName().color(NamedTextColor.RED).append(Component.text(" VS ").color(NamedTextColor.GOLD)).append(this.playerB.displayName().color(NamedTextColor.BLUE)), Component.empty()), onlinePlayers).runTaskLater(this.plugin, 80);
        new BetterBukkitRunnable(() -> {
            this.isWaiting = false;
            this.playerA.teleport(this.aSpawn);
            this.playerB.teleport(this.bSpawn);
            this.playerA.setHealth(20);
            this.playerB.setHealth(20);
            this.playerA.setFoodLevel(20);
            this.playerA.setSaturation(20);
            this.playerA.setExhaustion(0);
            this.playerB.setFoodLevel(20);
            this.playerB.setSaturation(20);
            this.playerB.setExhaustion(0);
            this.playerA.setGameMode(GameMode.SURVIVAL);
            this.playerB.setGameMode(GameMode.SURVIVAL);
        }).runTaskLater(this.plugin, 120);
    }
    public static void losePlayer(Player player){
        for (SumoGame game : games) {
            if (game.players.containsKey(player)) {
                Player other = null;
                Player win = null;
                if (player.equals(game.playerA)){
                    win = game.playerB;
                    other = game.playerA;
                    if (game.isWaiting){
                        win.teleport(game.bSpawn);
                        return;
                    }
                }
                if (player.equals(game.playerB)){
                    win = game.playerA;
                    other = game.playerB;
                    if (game.isWaiting){
                        win.teleport(game.aSpawn);
                        return;
                    }
                }
                if (other == null || win == null){
                    return;
                }
                win.showTitle(Title.title(Component.text("恭喜获胜！"), Component.empty()));
                game.isWaiting = true;
                for (Player onlinePlayer : game.server.getOnlinePlayers()) {
                    onlinePlayer.showTitle(Title.title(Component.text("恭喜" + win.getName() + "获胜").color(NamedTextColor.GREEN), Component.empty()));
                }
                PlayerDataManager instance = PlayerDataManager.getInstance();
                GlobalPlayerData playerData = instance.getPlayerData(other);
                playerData.addMoney(game.nowPrice+=100);
                other.setGameMode(GameMode.SPECTATOR);
                other.teleport(game.playerJoinLocation);
                game.players.get(other).setRank(game.nowRank--);
                game.round();
                return;
            }
        }
    }
    @Override
    public void close(){
        if (this.closed){
            return;
        }
        this.closed = true;
        multiverseCore.deleteWorld(this.playWorldName);
    }
}
