package cn.wzpmc.hny.pojo;

import cn.wzpmc.hny.HappyNewLobby;
import cn.wzpmc.hny.games.enums.Games;
import lombok.Getter;
import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.Server;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.block.data.BlockData;
import org.bukkit.block.data.Powerable;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.function.Consumer;

public class BoxPosition {
    @Getter
    private final Location pos1;
    @Getter
    private final Location pos2;
    private boolean sorted = false;
    public BoxPosition(Location pos1, Location pos2){
        this.pos1 = pos1;
        this.pos2 = pos2;
    }
    public BoxPosition(double x1, double y1, double z1, double x2, double y2, double z2){
        this(new Location(null, x1, y1, z1), new Location(null, x2, y2, z2));
    }

    public void sort(){
        if (sorted){
            return;
        }
        double maxX = Math.max(pos1.getX(), pos2.getX());
        double minX = Math.min(pos1.getX(), pos2.getX());
        double maxY = Math.max(pos1.getY(), pos2.getY());
        double minY = Math.min(pos1.getY(), pos2.getY());
        double maxZ = Math.max(pos1.getZ(), pos2.getZ());
        double minZ = Math.min(pos1.getZ(), pos2.getZ());
        this.pos2.setX(maxX);
        this.pos2.setY(maxY);
        this.pos2.setZ(maxZ);
        this.pos1.setX(minX);
        this.pos1.setY(minY);
        this.pos1.setZ(minZ);
        this.sorted = true;
    }
    public boolean isIn(Location pos){
        double x = pos.getX();
        double y = pos.getY();
        double z = pos.getZ();
        this.sort();
        if (x < this.pos1.getX() || x > this.pos2.getX()){
            return false;
        }
        if (y < this.pos1.getY() || y > this.pos2.getY()){
            return false;
        }
        return !(z < this.pos1.getZ()) && !(z > this.pos2.getZ());
    }
    private static final double step = 0.1;
    public List<Location> getEdgePointPoses(){
        this.sort();
        List<Location> result = new ArrayList<>();
        for (double i = pos1.getX(); i < pos2.getX(); i+=step) {
            result.add(new Location(null, i, pos1.getY(), pos1.getZ()));
            result.add(new Location(null, i, pos2.getY(), pos1.getZ()));
        }
        for (double i = pos1.getZ(); i < pos2.getZ(); i+=step) {
            result.add(new Location(null, pos2.getX(), pos1.getY(), i));
            result.add(new Location(null, pos2.getX(), pos2.getY(), i));
        }
        for (double i = pos2.getX(); i > pos1.getX(); i-=step) {
            result.add(new Location(null, i, pos1.getY(), pos2.getZ()));
            result.add(new Location(null, i, pos2.getY(), pos2.getZ()));
        }
        for (double i = pos2.getZ(); i > pos1.getZ(); i-=step) {
            result.add(new Location(null, pos1.getX(), pos1.getY(), i));
            result.add(new Location(null, pos1.getX(), pos2.getY(), i));
        }
        for (double i = pos1.getY(); i < pos2.getY(); i+=step) {
            result.add(new Location(null, pos1.getX(), i, pos1.getZ()));
            result.add(new Location(null, pos1.getX(), i, pos2.getZ()));
            result.add(new Location(null, pos2.getX(), i, pos1.getZ()));
            result.add(new Location(null, pos2.getX(), i, pos2.getZ()));
        }
        return result;
    }

    private record EdgeDescription(Edge edge, double edgePos) {
        private Location getNearestPointOnEdge(Location location) {
            Location result = location.clone();
            switch (edge) {
                case X_EDGE -> result.setX(edgePos);
                case Z_EDGE -> result.setZ(edgePos);
            }
            return result;
        }
        private double getDistanceOfPoint(Location location){
            return switch (edge) {
                case X_EDGE -> Math.abs(location.getX() - edgePos);
                case Z_EDGE -> Math.abs(location.getZ() - edgePos);
            };
        }
    }
    private enum Edge{
        X_EDGE, Z_EDGE
    }
    private Location getNearestEdgePoint(Location location, int downY, boolean isUp) {
        sort();
        List<EdgeDescription> edges = new ArrayList<>();
        edges.add(new EdgeDescription(Edge.X_EDGE, pos1.getX() - 1));
        edges.add(new EdgeDescription(Edge.X_EDGE, pos2.getX() - 1));
        edges.add(new EdgeDescription(Edge.Z_EDGE, pos1.getZ() - 1));
        edges.add(new EdgeDescription(Edge.Z_EDGE, pos2.getZ() - 1));
        edges.sort(Comparator.comparing(edge -> edge.getDistanceOfPoint(location)));
        for (EdgeDescription edge : edges) {
            Location nearestPointOnEdge = edge.getNearestPointOnEdge(location);
            Location nearestPointOnEdgeBlock = nearestPointOnEdge.add(0, downY - 1, 0);
            if (nearestPointOnEdgeBlock.getBlock().isEmpty()) {
                continue;
            }
            return nearestPointOnEdge.add(0, downY + 2, 0);
        }
        if (!isUp){
            if (location.getZ() - downY < 0){
                isUp = true;
                return getNearestEdgePoint(location, 1, isUp);
            }
            return getNearestEdgePoint(location, downY - 1, isUp);
        } else {
            return getNearestEdgePoint(location, downY + 1, isUp);
        }
    }
    public Location getNearestEdgePoint(Location location){
        return getNearestEdgePoint(location, 0, false);
    }
    public void replaceAir(World world, Material material){
        replace(world, Material.AIR, material);
    }
    public void replace(World world, Material original, Material target){
        forEachBlock(world, block -> {
            if (block.getType().equals(original)) {
                block.setType(target);
            }
        });
    }
    public void forEachBlock(World world, Consumer<Block> blockConsumer){
        sort();
        int x1 = pos1.getBlockX();
        int y1 = pos1.getBlockY();
        int z1 = pos1.getBlockZ();
        int x2 = pos2.getBlockX();
        int y2 = pos2.getBlockY();
        int z2 = pos2.getBlockZ();
        for (int x = x1; x <= x2; x++) {
            for (int y = y1; y <= y2; y++) {
                for (int z = z1; z <= z2; z++){
                    blockConsumer.accept(new Location(world, x, y, z).getBlock());
                }
            }
        }
    }
    public void triggerRedstoneLamp(World world, boolean powered){
        HappyNewLobby plugin = HappyNewLobby.getPlugin(HappyNewLobby.class);
        Server server = plugin.getServer();
        BlockData poweredData = server.createBlockData("minecraft:redstone_lamp[lit=" + powered + "]");
        this.forEachBlock(world, block -> {
            if (block.getType().equals(Material.REDSTONE_LAMP)){
                block.setBlockData(poweredData, false);
            }
        });
    }
}
