package cn.wzpmc.hny.pojo;

import cn.wzpmc.hny.games.enums.Teams;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;

import java.util.ArrayList;
import java.util.List;

@Getter
@AllArgsConstructor
public class GlobalPlayerData {
    private double money;
    private final List<CustomItemInfo> inventory;
    private int armorHeadLevel;
    private int armorLegLevel;
    private double erosionLevel;
    @Getter
    @Setter
    private Teams team;
    @Getter
    @Setter
    private boolean shield;
    public GlobalPlayerData(){
        this(0d, new ArrayList<>(), 0, 0, 0, Teams.ATTACK, false);
    }
    public void addMoney(double add){
        this.money += add;
    }
    public void reduceMoney(double reduce){
        this.money -= reduce;
    }
    public boolean addItem(CustomItem item){
        for (CustomItemInfo customItemInfo : inventory) {
            CustomItem type = customItemInfo.getType();
            if (type.equals(item)){
                int amount = customItemInfo.getAmount();
                if (amount >= 64){
                    return false;
                }
                customItemInfo.setAmount(amount + 1);
                return true;
            }
        }
        if (this.inventory.size() >= 26) {
            return false;
        }
        this.inventory.add(new CustomItemInfo(item, 1));
        return true;
    }
    public boolean removeItem(CustomItem item){
        return this.inventory.removeIf(e -> e.getType().equals(item));
    }
    public void reduceItem(CustomItem item){
        int amount = -1;
        for (CustomItemInfo customItemInfo : this.inventory) {
            if (customItemInfo.getType().equals(item)) {
                amount = customItemInfo.getAmount();
                if (amount > 1){
                    customItemInfo.setAmount(amount - 1);
                    return;
                }
                break;
            }
        }
        if (amount == 1){
            removeItem(item);
        }
    }
    public void addArmorHeadLevel(){
        if (this.armorHeadLevel == 5){
            return;
        }
        this.armorHeadLevel += 1;
    }
    public void addArmorLegLevel(){
        if (this.armorLegLevel == 5){
            return;
        }
        this.armorLegLevel += 1;
    }
    public void addErosionLevel(double value){
        this.erosionLevel += value;
        if (this.erosionLevel > 100){
            this.erosionLevel = 100;
        }
    }
    public void reduceErosionLevel(double value){
        this.erosionLevel -= value;
        if (this.erosionLevel < 0){
            this.erosionLevel = 0;
        }
    }

}
