package cn.wzpmc.hny.utils;

import cn.wzpmc.hny.HappyNewLobby;
import cn.wzpmc.hny.games.PlayerDataManager;
import cn.wzpmc.hny.games.PlayerEffectManager;
import cn.wzpmc.hny.games.enums.Teams;
import cn.wzpmc.hny.pojo.CustomItem;
import cn.wzpmc.hny.pojo.GlobalPlayerData;
import lombok.Getter;
import net.kyori.adventure.text.Component;
import org.bukkit.Location;
import org.bukkit.World;
import org.bukkit.block.Block;
import org.bukkit.entity.Entity;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.HumanEntity;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.InventoryClickEvent;
import org.bukkit.inventory.InventoryView;
import org.bukkit.inventory.PlayerInventory;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.util.RayTraceResult;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.function.Predicate;

import static cn.wzpmc.hny.pojo.CustomItem.ItemStatic.*;

public class ClickEventRegister {
    private static class EffectClickEvent extends ConsumeClickEvent {
        public EffectClickEvent(PlayerEffectManager.Effects effect, long t, TimeUnit unit) {
            super(player -> {
                PlayerEffectManager effectManager = PlayerEffectManager.getInstance();
                if (effectManager.hasEffect(player, effect)) {
                    return false;
                }
                effectManager.addEffect(player, effect, t, unit);
                return true;
            });
        }
        public EffectClickEvent(PlayerEffectManager.Effects effect) {
            this(effect, 1L, TimeUnit.DAYS);
        }
    }
    private static class ConsumeClickEvent implements ClickEvent {
        private final Predicate<Player> apply;
        public ConsumeClickEvent(Predicate<Player> apply){
            this.apply = apply;
        }
        @Override
            public void onClick(CustomItem item, InventoryClickEvent event) {
                InventoryView view = event.getView();
                HumanEntity human = view.getPlayer();
                if (human instanceof Player player) {
                    if (apply.test(player)) {
                        PlayerDataManager instance = PlayerDataManager.getInstance();
                        instance.getPlayerData(player).reduceItem(item);
                        instance.showPlayerInventory(player);
                        instance.savePlayerData(player);
                    }
                }
            }
        }
    @FunctionalInterface
    public interface ClickEvent {
        void onClick(CustomItem item, InventoryClickEvent event);
    }
    @Getter
    private static final Map<CustomItem, ClickEvent> clickEvents = new HashMap<>();
    public static void addEvent(CustomItem customItem, ClickEvent event){
        clickEvents.put(customItem, event);
    }
    static {
        addEvent(HEART, new EffectClickEvent(PlayerEffectManager.Effects.RAINBOW_HEALTH));
        addEvent(STRENGTH, new EffectClickEvent(PlayerEffectManager.Effects.POWER_WELDING));
        addEvent(BADLY_HURT, new EffectClickEvent(PlayerEffectManager.Effects.BADLY_HURT));
        addEvent(TOTEM_OF_UNDYING, new EffectClickEvent(PlayerEffectManager.Effects.TOTEM_OF_UNDYING));
        addEvent(TREAT, new ConsumeClickEvent(player -> {
            PlayerEffectManager effectManager = PlayerEffectManager.getInstance();
            if (effectManager.hasEffect(player, PlayerEffectManager.Effects.TREAT) || effectManager.hasEffect(player, PlayerEffectManager.Effects.BE_BADLY_HURT)) {
                player.sendMessage(Component.text("当前无法使用治疗道具"));
                return false;
            }
            effectManager.addEffect(player, PlayerEffectManager.Effects.TREAT, 10, TimeUnit.SECONDS);
            return true;
        }));
        addEvent(TURBINE, new EffectClickEvent(PlayerEffectManager.Effects.TURBINE, 5, TimeUnit.SECONDS));
        addEvent(MILK, new ConsumeClickEvent(player -> {PlayerEffectManager.getInstance().removeEffect(player);player.getActivePotionEffects().forEach(e -> player.removePotionEffect(e.getType())); return true;}));
        addEvent(EVOLUTION_CRYSTAL, new ConsumeClickEvent(player -> {
            PlayerDataManager instance = PlayerDataManager.getInstance();
            if (instance.getPlayerData(player).getTeam().equals(Teams.ATTACK)){
                player.sendMessage(Component.text("仅防守方可以使用！"));
                return false;
            }
            RayTraceResult rayTraceResult = player.rayTraceBlocks(10);
            if (rayTraceResult == null){
                player.sendMessage(Component.text("无法放置，请指向要放置的方块"));
                return false;
            }
            Block hitBlock = rayTraceResult.getHitBlock();
            if (hitBlock == null){
                player.sendMessage(Component.text("无法放置，请指向要放置的方块"));
                return false;
            }
            Location clone = hitBlock.getLocation().clone();
            clone.add(0,1,0);
            World world = clone.getWorld();
            Entity entity = world.spawnEntity(clone, EntityType.ENDER_CRYSTAL);
            PersistentDataContainer persistentDataContainer = entity.getPersistentDataContainer();
            UUID uniqueId = player.getUniqueId();
            persistentDataContainer.set(EVOLUTION_CRYSTAL_PLAYER_KEY, PersistentDataType.LONG_ARRAY, new long[]{uniqueId.getMostSignificantBits(), uniqueId.getLeastSignificantBits()});
            entity.customName(Component.text("进化水晶"));
            return true;
        }));
        addEvent(PILL, new ConsumeClickEvent(player -> {
            PlayerDataManager instance = PlayerDataManager.getInstance();
            GlobalPlayerData playerData = instance.getPlayerData(player);
            double erosionLevel = playerData.getErosionLevel();
            if (erosionLevel <= 0){
                player.sendMessage(Component.text("你目前的侵蚀度为0"));
                return false;
            }
            if (erosionLevel >= 95){
                player.sendMessage(Component.text("你已经被完全侵蚀了，解药对你无效"));
                return false;
            }
            playerData.reduceErosionLevel(15);
            instance.savePlayerData(player);
            return true;
        }));
        addEvent(PUT_OFF, new EffectClickEvent(PlayerEffectManager.Effects.PUT_OFF));
        addEvent(CODE, new EffectClickEvent(PlayerEffectManager.Effects.CODE));
        addEvent(SEED, new ConsumeClickEvent(player -> {
            HappyNewLobby plugin = HappyNewLobby.getPlugin(HappyNewLobby.class);
            for (int i = 0; i < 12000; i+= 1200) {
                new BetterBukkitRunnable(() -> {
                    PlayerDataManager instance = PlayerDataManager.getInstance();
                    instance.getPlayerData(player).addMoney(10);
                    instance.savePlayerData(player);
                }).runTaskLater(plugin, i);
            }
            return true;
        }));
    }
}
