package cn.wzpmc.hny.utils;

import com.destroystokyo.paper.ParticleBuilder;
import org.bukkit.Color;
import org.bukkit.Location;
import org.bukkit.Particle;
import org.mmga.utils.utils.map.IncreaseAbleHashMap;

public class ParticleUtils {
    private record CircleData(Location location, double radius, Color color){}
    private static final IncreaseAbleHashMap<CircleData, ParticleBuilder> cache = new IncreaseAbleHashMap<>();
    /**
     * 使用粒子效果绘制并填充一个圆
     * @param location 圆心位置
     * @param radius 半径
     * @param color 粒子效果的颜色
     */
    public static void drawCircleParticle(Location location, double radius, Color color) {
        CircleData circleData = new CircleData(location, radius, color);
        if (cache.containsKey(circleData)) {
            for (ParticleBuilder builder : cache.get(circleData)) {
                builder.spawn();
            }
            return;
        }
        final int points = 50; // 用于圆边界的点数
        final double increment = (2 * Math.PI) / points; // 边界上每个点之间的角度增量

        // 填充圆内部
        for (double r = 0; r <= radius; r += 0.5) { // 逐步减小半径以填充内部
            int fillPoints = points;
            double fillIncrement = increment;
            if (r <= radius){
                fillPoints = (int) (points * (r / radius)); // 根据当前半径调整点数
                fillIncrement = (2 * Math.PI) / fillPoints;
            }
            for (int i = 0; i < fillPoints; i++) {
                double angle = i * fillIncrement;
                double x = location.getX() + (r * Math.cos(angle));
                double z = location.getZ() + (r * Math.sin(angle));
                Location particleLocation = new Location(location.getWorld(), x, location.getY(), z);
                ParticleBuilder particleBuilder = new ParticleBuilder(Particle.DUST_COLOR_TRANSITION)
                        .data(new Particle.DustTransition(color, color, 1))
                        .location(particleLocation);
                cache.add(circleData, particleBuilder);
                particleBuilder.spawn();
            }
        }
    }
}
